<?php

require __DIR__ . '/vendor/autoload.php';

use net\authorize\api\contract\v1 as AnetAPI;
use net\authorize\api\controller as AnetController;
class WPBS_AuthorizeNet_API
{
    public static function keys()
    {
        $settings = get_option('wpbs_settings', array());
        $authorize_net_api = get_option('wpbs_authorize_net_api', array());
        $environment = ((isset($settings['payment_authorize_net_sandbox']) && $settings['payment_authorize_net_sandbox'] == 'on')) ? 'sandbox' : 'live';

        if ($environment == 'sandbox') {
            return array(
                'environment' => $environment,
                'api_login_id' => isset($authorize_net_api['payment_authorize_net_sandbox_api_login_id']) ? $authorize_net_api['payment_authorize_net_sandbox_api_login_id'] : '',
                'transaction_key' => isset($authorize_net_api['payment_authorize_net_sandbox_transaction_key']) ? $authorize_net_api['payment_authorize_net_sandbox_transaction_key'] : '',
                'client_key' => isset($authorize_net_api['payment_authorize_net_sandbox_client_key']) ? $authorize_net_api['payment_authorize_net_sandbox_client_key'] : '',
            );
        }

        return array(
            'environment' => $environment,
            'api_login_id' => isset($authorize_net_api['payment_authorize_net_live_api_login_id']) ? $authorize_net_api['payment_authorize_net_live_api_login_id'] : '',
            'transaction_key' => isset($authorize_net_api['payment_authorize_net_live_transaction_key']) ? $authorize_net_api['payment_authorize_net_live_transaction_key'] : '',
            'client_key' => isset($authorize_net_api['payment_authorize_net_live_client_key']) ? $authorize_net_api['payment_authorize_net_live_client_key'] : '',
        );

    }

    public static function script()
    {
        $accept_ui_script = array(
            'sandbox' => 'https://jstest.authorize.net/v1/Accept.js',
            'live' => 'https://js.authorize.net/v1/Accept.js',
        );

        $api = self::keys();

        return $accept_ui_script[$api['environment']];
    }
}

class WPBS_AuthorizeNet
{
    public static function createAnAcceptPaymentTransaction($amount, $dataValue, $invoice_number, $item_description)
    {

        $api = WPBS_AuthorizeNet_API::keys();

        $merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
        $merchantAuthentication->setName($api['api_login_id']);
        $merchantAuthentication->setTransactionKey($api['transaction_key']);

        // Set the transaction's refId
        $refId = 'ref' . time();

        // Create the payment object for a payment nonce
        $opaqueData = new AnetAPI\OpaqueDataType();
        $opaqueData->setDataDescriptor("COMMON.ACCEPT.INAPP.PAYMENT");
        $opaqueData->setDataValue($dataValue);

        // Add the payment data to a paymentType object
        $paymentOne = new AnetAPI\PaymentType();
        $paymentOne->setOpaqueData($opaqueData);

        // Create order information
        $order = new AnetAPI\OrderType();
        $order->setInvoiceNumber($invoice_number);
        $order->setDescription($item_description);

        // Create a TransactionRequestType object and add the previous objects to it
        $transactionRequestType = new AnetAPI\TransactionRequestType();
        $transactionRequestType->setTransactionType("authCaptureTransaction");
        $transactionRequestType->setAmount($amount);
        $transactionRequestType->setOrder($order);
        $transactionRequestType->setPayment($paymentOne);

        // Assemble the complete transaction request
        $request = new AnetAPI\CreateTransactionRequest();
        $request->setMerchantAuthentication($merchantAuthentication);
        $request->setRefId($refId);
        $request->setTransactionRequest($transactionRequestType);

        // Create the controller and get the response
        $controller = new AnetController\CreateTransactionController($request);

        if ($api['environment'] == 'sandbox') {
            $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::SANDBOX);
        } else {
            $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::PRODUCTION);
        }

        $nice_response = array();

        if ($response != null) {
            // Check to see if the API request was successfully received and acted upon
            if ($response->getMessages()->getResultCode() == "Ok") {
                // Since the API request was successful, look for a transaction response
                // and parse it to display the results of authorizing the card
                $tresponse = $response->getTransactionResponse();

                if ($tresponse != null && $tresponse->getMessages() != null) {
                    $nice_response['status'] = 'success';
                    $nice_response['transaction_id'] = $tresponse->getTransId();
                    $nice_response['response_code'] = $tresponse->getResponseCode();
                    $nice_response['message_code'] = $tresponse->getMessages()[0]->getCode();
                    $nice_response['auth_code'] = $tresponse->getAuthCode();
                    $nice_response['description'] = $tresponse->getMessages()[0]->getDescription();

                } else {
                    if ($tresponse->getErrors() != null) {
                        $nice_response['status'] = 'error';
                        $nice_response['error_code'] = $tresponse->getErrors()[0]->getErrorCode();
                        $nice_response['error_message'] = $tresponse->getErrors()[0]->getErrorText();
                    }
                }
                // Or, print errors if the API request wasn't successful
            } else {
                $tresponse = $response->getTransactionResponse();

                if ($tresponse != null && $tresponse->getErrors() != null) {
                    $nice_response['status'] = 'error';
                    $nice_response['error_code'] = $tresponse->getErrors()[0]->getErrorCode();
                    $nice_response['error_message'] = $tresponse->getErrors()[0]->getErrorText();
                } else {
                    $nice_response['status'] = 'error';
                    $nice_response['error_code'] = $response->getMessages()->getMessage()[0]->getCode();
                    $nice_response['error_message'] = $response->getMessages()->getMessage()[0]->getText();
                }
            }
        }

        $nice_response['raw'] = $response;

        return $nice_response;
    }

    public static function getTransactionDetails($transactionId)
    {
        $api = WPBS_AuthorizeNet_API::keys();

        $merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
        $merchantAuthentication->setName($api['api_login_id']);
        $merchantAuthentication->setTransactionKey($api['transaction_key']);

        $request = new AnetAPI\GetTransactionDetailsRequest();
        $request->setMerchantAuthentication($merchantAuthentication);
        $request->setTransId($transactionId);

        $controller = new AnetController\GetTransactionDetailsController($request);

        if ($api['environment'] == 'sandbox') {
            $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::SANDBOX);
        } else {
            $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::PRODUCTION);
        }

        $nice_response = array();

        $nice_response['raw'] = $response;

        if (($response != null) && ($response->getMessages()->getResultCode() == "Ok")) {
            $nice_response['status'] = $response->getTransaction()->getTransactionStatus();
            $nice_response['amount'] = $response->getTransaction()->getAuthAmount();
            $nice_response['transaction_id'] = $response->getTransaction()->getTransId();
            $nice_response['credit_card_number'] = $response->getTransaction()->getPayment()->getCreditCard()->getCardNumber();
            $nice_response['credit_card_exp'] = $response->getTransaction()->getPayment()->getCreditCard()->getExpirationDate();
            $nice_response['credit_card_type'] = $response->getTransaction()->getPayment()->getCreditCard()->getCardType();

        } else {
            $errorMessages = $response->getMessages()->getMessage();
            $nice_response['status'] = 'error';
            $nice_response['error_code'] = $errorMessages[0]->getCode();
            $nice_response['error_message'] = $errorMessages[0]->getText();
        }

        return $nice_response;

    }

    public static function refundTransaction($charge, $amount)
    {
        $api = WPBS_AuthorizeNet_API::keys();

        $merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
        $merchantAuthentication->setName($api['api_login_id']);
        $merchantAuthentication->setTransactionKey($api['transaction_key']);
        // Set the transaction's refId

        $refId = 'ref' . time();

        $refTransId = $charge['transaction_id'];

        // Create the payment data for a credit card
        $creditCard = new AnetAPI\CreditCardType();
        $creditCard->setCardNumber($charge['credit_card_number']);
        $creditCard->setExpirationDate($charge['credit_card_exp']);
        $paymentOne = new AnetAPI\PaymentType();
        $paymentOne->setCreditCard($creditCard);

        //create a transaction
        $transactionRequest = new AnetAPI\TransactionRequestType();
        $transactionRequest->setTransactionType("refundTransaction");
        $transactionRequest->setAmount($amount);
        $transactionRequest->setPayment($paymentOne);
        $transactionRequest->setRefTransId($refTransId);

        $request = new AnetAPI\CreateTransactionRequest();
        $request->setMerchantAuthentication($merchantAuthentication);
        $request->setRefId($refId);
        $request->setTransactionRequest($transactionRequest);
        $controller = new AnetController\CreateTransactionController($request);

        if ($api['environment'] == 'sandbox') {
            $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::SANDBOX);
        } else {
            $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::PRODUCTION);
        }

        if ($response != null) {
            if ($response->getMessages()->getResultCode() == "Ok") {
                $tresponse = $response->getTransactionResponse();

                if ($tresponse != null && $tresponse->getMessages() != null) {

                    return ['success' => true, 'response' => [
                        'code' => $tresponse->getResponseCode(),
                        'transaction_id' => $tresponse->getTransId(),
                        'transaction' => $tresponse,
                        'resposne' => $response,

                    ]];
                } else {
                    if ($tresponse->getErrors() != null) {
                        return ['error' => true, 'error_message' => $tresponse->getErrors()[0]->getErrorCode() . ': ' . $tresponse->getErrors()[0]->getErrorText()];
                    }
                }
            } else {
                $tresponse = $response->getTransactionResponse();
                if ($tresponse != null && $tresponse->getErrors() != null) {
                    return ['error' => true, 'error_message' => $tresponse->getErrors()[0]->getErrorCode() . ': ' . $tresponse->getErrors()[0]->getErrorText()];
                } else {
                    return ['error' => true, 'error_message' => $response->getMessages()->getMessage()[0]->getCode() . ': ' . $response->getMessages()->getMessage()[0]->getText()];
                }
            }
        } else {
            return ['error' => true, 'error_message' => 'no response returned'];
        }

        return $response;
    }
}
